--*****************************************************************************
--*  Copyright (c) 2011 by Michael Fischer. All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  10.12.2011  mifi  First Version
--*****************************************************************************

--/*=========================================================================*/
--/*  DEFINE: Library                                                        */
--/*=========================================================================*/
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;


--/*=========================================================================*/
--/*  DEFINE: Entity                                                         */
--/*=========================================================================*/

entity de0_nano_chibios is
  port( 
        --
        -- Input clock 
        --
        CLOCK_50      : in  std_logic;
        
        --
        -- SDRAM interface,
        -- here a A3V64S40ETP-G6 (166MHz@CL-3) is used.
        -- Reference is made to Zentel datasheet:
        -- A3V64S40ETP, Revision 1.2, Mar., 2010
        --        
        SDRAM_CLK     : out   std_logic;                       -- Master Clock
        SDRAM_CKE     : out   std_logic;                       -- Clock Enable    
        SDRAM_CS_N    : out   std_logic;                       -- Chip Select
        SDRAM_RAS_N   : out   std_logic;                       -- Row Address Strobe
        SDRAM_CAS_N   : out   std_logic;                       -- Column Address Strobe
        SDRAM_WE_N    : out   std_logic;                       -- Write Enable
        SDRAM_DQ      : inout std_logic_vector(15 downto 0);   -- Data I/O (16 bits)
        SDRAM_DQML    : out   std_logic;                       -- Output Disable / Write Mask (low)
        SDRAM_DQMU    : out   std_logic;                       -- Output Disable / Write Mask (high)
        SDRAM_ADDR    : out   std_logic_vector(12 downto 0);   -- Address Input (12 bits)
        SDRAM_BA_0    : out   std_logic;                       -- Bank Address 0
        SDRAM_BA_1    : out   std_logic;                       -- Bank Address 1
        
        --
        -- LEDs, green and heartbeat
        --
        LED_GREEN     : out   std_logic_vector(6 downto 0);
        LED_HEARTBEAT : out   std_logic;
        
        --
        -- Swithes and keys
        --
        KEY           : in    std_logic_vector(1 downto 0);
        SW            : in    std_logic_vector(3 downto 0)
        
      );
end entity de0_nano_chibios;


--/*=========================================================================*/
--/*  DEFINE: Architecture                                                   */
--/*=========================================================================*/

architecture syn of de0_nano_chibios is

   --
   -- Define all components which are included here
   --
   component pll_sys
     port( inclk0   : in  std_logic  := '0';
           c0       : out std_logic ;
           c1       : out std_logic ;
           c2       : out std_logic ;
           locked   : out std_logic 
         );
   end component pll_sys;
   
   
   component simple_counter
      port( clock       : in  std_logic;
            counter_out : out std_logic
          );
   end component simple_counter;   
   

   component de0_nano_sopc
      port (
            -- 1) global signals:
               signal clk_sys : IN STD_LOGIC;
               signal reset_n : IN STD_LOGIC;

            -- the_pio_key
               signal in_port_to_the_pio_key : IN STD_LOGIC_VECTOR (1 DOWNTO 0);

            -- the_pio_led_green
               signal out_port_from_the_pio_led_green : OUT STD_LOGIC_VECTOR (6 DOWNTO 0);

            -- the_pio_sw
               signal in_port_to_the_pio_sw : IN STD_LOGIC_VECTOR (3 DOWNTO 0);

            -- the_sdram
               signal zs_addr_from_the_sdram : OUT STD_LOGIC_VECTOR (12 DOWNTO 0);
               signal zs_ba_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
               signal zs_cas_n_from_the_sdram : OUT STD_LOGIC;
               signal zs_cke_from_the_sdram : OUT STD_LOGIC;
               signal zs_cs_n_from_the_sdram : OUT STD_LOGIC;
               signal zs_dq_to_and_from_the_sdram : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
               signal zs_dqm_from_the_sdram : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
               signal zs_ras_n_from_the_sdram : OUT STD_LOGIC;
               signal zs_we_n_from_the_sdram : OUT STD_LOGIC
            );
   end component de0_nano_sopc;
   
      
   --
   -- Define all local signals (like static data) here
   --
   signal clk_10     : std_logic;
   signal clk_sys    : std_logic;
   signal pll_locked : std_logic;
	signal sdram_ba   : std_logic_vector(1 downto 0);
	signal sdram_dqm  : std_logic_vector(1 downto 0);
  
begin

   inst_pll_sys : pll_sys
      port map ( inclk0 => CLOCK_50,
                 c0     => clk_sys,
                 c1     => SDRAM_CLK,
                 c2     => clk_10,
                 locked => pll_locked
               );
                                  
   inst_simple_counter : simple_counter 
      port map ( clock       => clk_10,
                 counter_out => LED_HEARTBEAT
               );   
               
   inst_cpu : de0_nano_sopc
      port map ( -- global signals:
                 clk_sys => clk_sys,
                 reset_n => pll_locked,
                 
                 -- the_pio_key
                 in_port_to_the_pio_key => KEY,

                 -- the_pio_led_green
                 out_port_from_the_pio_led_green => LED_GREEN,

                 -- the_pio_sw
                 in_port_to_the_pio_sw => SW,

                 -- the_sdram
                 zs_addr_from_the_sdram      => SDRAM_ADDR,
                 zs_ba_from_the_sdram        => sdram_ba,
                 zs_cas_n_from_the_sdram     => SDRAM_CAS_N,
                 zs_cke_from_the_sdram       => SDRAM_CKE,
                 zs_cs_n_from_the_sdram      => SDRAM_CS_N,
                 zs_dq_to_and_from_the_sdram => SDRAM_DQ,
                 zs_dqm_from_the_sdram       => sdram_dqm,
                 zs_ras_n_from_the_sdram     => SDRAM_RAS_N,
                 zs_we_n_from_the_sdram      => SDRAM_WE_N
               );  
				
   SDRAM_BA_1 <= sdram_ba(1);
	SDRAM_BA_0 <= sdram_ba(0);
	
	SDRAM_DQMU <= sdram_dqm(1);
	SDRAM_DQML <= sdram_dqm(0);	

end architecture syn;

-- *** EOF ***

